#include "packet.h"
#include "setcontrol.h"
#include "commands.h"

#include <sys/types.h>

static int next_msg_id = 0;

int SendSetControl()
{
	struct metadata m;
	struct controller_message cm;

	cm.id = ROLL_ID;        /* Roll controller */
	cm.value_id = KI_ID;    /* I coefficient */
	cm.value = 42.00f;      /* An appropriate value */

	uint8_t data[64];
	m.msg_id = next_msg_id++;
	/* Fills in the rest of the metadata automatically */
	if (EncodeSetcontrol(&m, data, 64, &cm) < 0) {
		return -1;
	}

	uint8_t packet[128];
	ssize_t length;
	if ((length = EncodePacket(packet, 128, &m, data)) < 0) {
		return -1;
	}

	/* Now you can send the packet
	 * sendThePacket(packet, length);
	 */

	return 0;
}


/* Process incoming packets */
int receive_packet(uint8_t * packet, size_t length)
{
	struct metadata m;
	uint8_t data[128];

	/* Will fail if checksum is bad, etc */
	if (DecodePacket(&m, data, 128, packet, length) < 0) {
		return -1;
	}

	/* Let's pretend that the callbacks have signature
	 * void cb(
	 *     const struct metadata * m,
	 *     const uint8_t * data);
	 *
	 * Call the callback.
	 */
	(*MessageTypes[m.msg_type].functionPtr)(&m, data);

	return 0;
}

/* An example of a setcontrol callback. Arguments
 * aren't important, as long as we have access
 * to the packet bytes and the length
 */
void cb_setcontrol(const struct metadata *m, const uint8_t *data)
{
	struct controller_message cm;
	DecodeSetcontrol(&cm, m, data);

	/* cm now has populated controller ID, value ID, and value */
}
